@@ -649,6 +649,15 @@ midori_search_engines_modify_cb (KatzeArray* array,
     g_free (config_file);
 }
 
+static void
+midori_search_engines_move_item_cb (KatzeArray* array,
+                                    gpointer    item,
+                                    gint        position,
+                                    KatzeArray* search_engines)
+{
+    midori_search_engines_modify_cb (array, item, search_engines);
+}
+
 static void
 midori_trash_add_item_cb (KatzeArray* trash,
                           GObject*    item)
@@ -762,7 +771,7 @@ midori_app_quit_cb (MidoriBrowser* browser,
     g_unlink (config_file);
     g_free (config_file);
 
-    if (save_timeout && session)
+    if (session)
         midori_session_save_timeout_cb (session);
 }
 
@@ -920,6 +929,13 @@ midori_soup_session_prepare (SoupSession*       session,
                      "SSL certificates cannot be verified."));
     #endif
 
+    #if !WEBKIT_CHECK_VERSION (1, 3, 5)
+    /* See http://stevesouders.com/ua/index.php */
+    g_object_set (session, "max-conns", 60,
+                           "max-conns-per-host", 6,
+                           NULL);
+    #endif
+
     soup_session_settings_notify_http_proxy_cb (settings, NULL, session);
     g_signal_connect (settings, "notify::http-proxy",
         G_CALLBACK (soup_session_settings_notify_http_proxy_cb), session);
@@ -1288,6 +1304,7 @@ midori_load_session (gpointer data)
     KATZE_ARRAY_FOREACH_ITEM (item, _session)
     {
         g_object_set_data (G_OBJECT (item), "midori-view-append", (void*)1);
+        katze_item_set_meta_integer (item, "dont-write-history", 1);
         midori_browser_add_item (browser, item);
     }
     current = katze_item_get_meta_integer (KATZE_ITEM (_session), "current");
@@ -1450,6 +1467,135 @@ signal_handler (int signal_id)
 }
 #endif
 
+#if 0
+static void
+midori_speeddial_import_from_json (const gchar* json_file,
+                                   const gchar* speeddial_file)
+{
+    guint i = 0;
+    guint columns = 3;
+    guint slot_count = 0;
+    gchar* json_content;
+    gchar** parts;
+    GKeyFile* key_file = g_key_file_new ();
+
+    g_file_get_contents (json_file, &json_content, NULL, NULL);
+    parts = g_strsplit (json_content ? json_content : "", ",", -1);
+    while (parts && parts[i] != NULL)
+    {
+        gchar* key;
+        gchar* val;
+        gchar* slot;
+        gchar* dial_id;
+        gchar* uri;
+        gchar** values = g_strsplit (parts[i], "\"", -1);
+
+        if (*values[1])
+        {
+            if (!g_strcmp0 (values[1], "shortcuts"))
+            {
+                key = g_strdup (values[3]);
+                val = g_strdup (values[5]);
+            }
+            else if (!g_strcmp0 (values[1], "thumb"))
+            {
+                key = g_strdup (values[1]);
+                val = g_strdup (values[2]);
+            }
+            else
+            {
+                key = g_strdup (values[1]);
+                val = g_strdup (values[3]);
+            }
+
+            if (g_str_equal (key, "id"))
+            {
+                /* FIXME: leaked string */
+                slot = g_strdup (val);
+                dial_id = g_strdup_printf ("Dial %s", slot + 1);
+                slot_count++;
+            }
+            else if (g_str_equal (key, "href") && (*val && strncmp (val, "#", 1)))
+            {
+                uri = g_strdup (val);
+                g_key_file_set_value (key_file, dial_id, "name", slot);
+                g_key_file_set_value (key_file, dial_id, "uri", uri);
+            }
+            else if (g_str_equal (key, "img") && *val)
+            {
+                gsize sz;
+                gint state = 0;
+                guint save = 0;
+                gchar* checksum;
+                gchar* filename;
+                gchar* thumb_dir;
+                gchar* thumb_path;
+                gsize base64_size = strlen (val);
+                guchar* decoded = g_malloc0 ((base64_size * 3) / 4);
+
+                sz = g_base64_decode_step (g_strdup (val), base64_size,
+                                           decoded, &state, &save);
+                checksum = g_compute_checksum_for_string (G_CHECKSUM_MD5, slot, -1);
+                filename  = g_strdup_printf ("%s%s", checksum, ".png");
+                g_free (checksum);
+                thumb_dir = g_build_path (G_DIR_SEPARATOR_S, g_get_user_cache_dir (),
+                                          PACKAGE_NAME, "thumbnails", NULL);
+                if (!g_file_test (thumb_dir, G_FILE_TEST_EXISTS))
+                    katze_mkdir_with_parents (thumb_dir, 0700);
+                thumb_path = g_build_filename (thumb_dir, filename, NULL);
+                g_free (filename);
+                g_file_set_contents (thumb_path, (gchar*)decoded, sz, NULL);
+
+                g_free (decoded);
+                g_free (thumb_dir);
+                g_free (thumb_path);
+            }
+            else if (g_str_equal (key, "thumb"))
+            {
+                guint thumb_size;
+                gchar* thumb_size_type;
+
+                /* FIXME: leaked string */
+                thumb_size = atoi (g_strndup (val + 1, strlen (val) - 3));
+                if (thumb_size == 80)
+                    thumb_size_type = g_strdup ("SMALL");
+                else if (thumb_size == 160)
+                    thumb_size_type = g_strdup ("MEDIUM");
+                else if (thumb_size == 240)
+                    thumb_size_type = g_strdup ("BIG");
+                g_key_file_set_value (key_file, "settings", "size", thumb_size_type);
+
+                g_free (thumb_size_type);
+            }
+            else if (g_str_equal (key, "title") && *val)
+            {
+                g_key_file_set_value (key_file, dial_id, "name", slot);
+                g_key_file_set_value (key_file, dial_id, key, val + 3);
+            }
+            else if (g_str_equal (key, "width"))
+            {
+                columns = atoi (val);
+            }
+
+            i++;
+
+            g_free (key);
+            g_free (val);
+            g_strfreev (values);
+        }
+    }
+
+    g_key_file_set_integer (key_file, "settings", "columns", columns);
+    g_key_file_set_integer (key_file, "settings", "rows", slot_count / columns);
+
+    sokoke_key_file_save_to_file (key_file, speeddial_file, NULL);
+
+    g_strfreev (parts);
+    g_free (json_content);
+    g_key_file_free (key_file);
+}
+#endif
+
 static void
 midori_soup_session_block_uris_cb (SoupSession* session,
                                    SoupMessage* msg,
@@ -1563,18 +1709,21 @@ midori_clear_web_cookies_cb (void)
     SoupSession* session = webkit_get_default_session ();
     SoupSessionFeature* jar = soup_session_get_feature (session, SOUP_TYPE_COOKIE_JAR);
     GSList* cookies = soup_cookie_jar_all_cookies (SOUP_COOKIE_JAR (jar));
+    SoupSessionFeature* feature;
+
     for (; cookies != NULL; cookies = g_slist_next (cookies))
     {
         SoupCookie* cookie = cookies->data;
-        soup_cookie_set_max_age (cookie, 0);
-        soup_cookie_free (cookie);
+        soup_cookie_jar_delete_cookie ((SoupCookieJar*)jar, cookie);
     }
-    g_slist_free (cookies);
+    soup_cookies_free (cookies);
     /* Removing KatzeHttpCookies makes it save outstanding changes */
-    if (soup_session_get_feature (session, KATZE_TYPE_HTTP_COOKIES))
+    if ((feature = soup_session_get_feature (session, KATZE_TYPE_HTTP_COOKIES)))
     {
-        soup_session_remove_feature_by_type (session, KATZE_TYPE_HTTP_COOKIES);
-        soup_session_add_feature_by_type (session, KATZE_TYPE_HTTP_COOKIES);
+        g_object_ref (feature);
+        soup_session_remove_feature (session, feature);
+        soup_session_add_feature (session, feature);
+        g_object_unref (feature);
     }
 }
 
@@ -1654,6 +1803,9 @@ main (int    argc,
     MidoriWebSettings* settings;
     gchar* config_file;
     gchar* bookmarks_file;
+#if 0
+    gchar* speeddial_file;
+#endif
     gboolean bookmarks_exist;
     MidoriStartup load_on_startup;
     KatzeArray* search_engines;
@@ -1866,7 +2018,12 @@ main (int    argc,
                           NULL);
             midori_browser_set_action_visible (browser, "Menubar", FALSE);
         }
-        g_object_set (settings, "show-panel", FALSE, NULL);
+        g_object_set (settings, "show-panel", FALSE,
+                      "last-window-state", MIDORI_WINDOW_NORMAL,
+                      #if WEBKIT_CHECK_VERSION (1, 1, 2)
+                      "enable-private-browsing", TRUE,
+                      #endif
+                      NULL);
         midori_browser_set_action_visible (browser, "Tools", FALSE);
         midori_browser_set_action_visible (browser, "Panel", FALSE);
         g_object_set (browser, "settings", settings, NULL);
@@ -2060,6 +2217,17 @@ main (int    argc,
     g_free (bookmarks_file);
     midori_startup_timer ("History read: \t%f");
 
+    #if 0
+    speeddial_file = g_build_filename (config, "speeddial", NULL);
+    if (g_access (speeddial_file, F_OK) != 0)
+    {
+        gchar* json_file = g_build_filename (config, "speeddial.json", NULL);
+        midori_speeddial_import_from_json (json_file, speeddial_file);
+        g_free (json_file);
+    }
+    g_free (speeddial_file);
+    #endif
+
     /* In case of errors */
     if (error_messages->len)
     {
@@ -2135,6 +2303,8 @@ main (int    argc,
             KATZE_ARRAY_FOREACH_ITEM (item, search_engines)
                 g_signal_connect_after (item, "notify",
                     G_CALLBACK (midori_search_engines_modify_cb), search_engines);
+            g_signal_connect_after (search_engines, "move-item",
+                G_CALLBACK (midori_search_engines_move_item_cb), search_engines);
         }
     }
     g_signal_connect_after (trash, "add-item",
@@ -2200,6 +2370,8 @@ main (int    argc,
     gtk_main ();
 
     settings = katze_object_get_object (app, "settings");
+    settings_notify_cb (settings, NULL, app);
+
     g_object_get (settings, "maximum-history-age", &max_history_age, NULL);
     midori_history_terminate (history, max_history_age);
     /* Removing KatzeHttpCookies makes it save outstanding changes */
